C program compare_all.f
C..............................................................................
C
C Program to compare a set of full & reduced chemical networks
C
C..............................................................................
C
C copyright (c) 2002, Dmitry A. Semenov, AIU Jena
C copyright (c) 2004-2009, Dmitry A. Semenov, MPIA Heidelberg
C
C..............................................................................
C
C Input parameter(s):  
C
C inpf        == a name of the file contains obtained relative abundances of 
C                chemical species in the case of a full chemical network,
C
C inpr        == a name of the file contains obtained relative abundances of 
C                chemical species in the case of a reduced chemical network,
C
C istart      == starting grid cell to be compared,
C
C iend        == final grid cell,
C
C out         == a name of the file with comparison of the inputs,
C
C vispecies   == species for which comparison will be done,
C
C rmin        == minimum relative abundance required
C
C..............................................................................
C
C Output file(s):
C
C out         == a name of the file with comparison of the inputs
C
C..............................................................................
C
C Global parameter(s):
C
C nreac       == maximal amount of chemical reactions to be read,
C
C nspec       == maximal amount of chemical species to be considered,
C
C ntime       == maximal amount of taken time steps
C 
C..............................................................................
C
C Important variable(s):
C
C rates1,     == a name of the rate file has been used in the computations,
C rates2
C
C tempg1,     == a gas temperature (K) has been used in calculations,
C tempg2
C
C rhog1,      ==  a gas density (g/cm^3) has been used in calculations,
C rhog2
C
C tempd1,     == a dust temperature (K) has been used in calculations,
C tempd2
C
C mdmg1,      == dust-to-gas mass ratio has been used in calculations,
C mdmg2
C
C grain1,     == a particle radius [cm] has been used in calculations,
C grain2
C
C rhod1,      == a dust grain density [g/cm^3] has been used in calculations,
C rhod2
C
C Av1,        == visual extinction (mag) has been used in calculations,
C Av2
C
C rad1,       == enhancement of the local radiation field in comparison with
C rad2           the interstellar one,
C 
C zeta1,      == cosmic ray ionisation rate,
C zeta2
C
C albedo_UV1, == albedo of dust grains in far UV (~150 nm)
C albedo_UV2
C
C tlast1,     == evolution time (years),
C tlast2 
C
C tstep1,     == first time step (years),
C tstep2
C
C s0_1(ns0_1),== an array of names of species for those initial abundances are
C s0_2(ns0_2)    not zero,
C
C rab0_1(ns0_1),== initial abundances of the species, relative to n(H+2H2), 
C rab0_2(ns0_2)
C
C ny1,        == amount of non-conserved species in the species set,
C ny2
C 
C s1(ny1),== a species set,
C s2(ny2)
C
C time1(nt1), == an array of time steps (years),
C time2(nt2)
C
C ak1(nr1),   == an array of rate coefficients,
C ak2(nr2)
C
C rab1(ny1,== calculated abundances of the species, relative to n(H+2H2),
C      nt1),
C rab2(ny2,
C      nt2)   
C
C eps         == possible relative deviation in the input data,
C
C rabr_max(   == maximal abundances of species,
C     ny2)
C
C is(ny2) == indices of species,
C
C js(ny2) == indices of species,
C
C..............................................................................
C
C Used subroutine(s) (alphabetically): len_tri2, maxarr
C 
C..............................................................................
      PROGRAM compare_all
      IMPLICIT NONE

C Global parameter(s):
      INTEGER nreac, nspec, ntime
      PARAMETER (nreac=9999, nspec=999, ntime=999)

C Global variable(s):
      INTEGER ns0_1, ns0_2, ny1, ny2, nt1, nt2, nr1, nr2, iter
      REAL*8 tempg1, rhog1, mdmg1, grain1, rhod1, AvSt1, AvIS1, G01, 
     1       zeta1, albedo_UV1, tlast1, tstep1, rab0_1, time1, rab1,ak1,
     2       tempg2, rhog2, mdmg2, grain2, rhod2, AvSt2, AvIS2, G02, 
     3       zeta2, albedo_UV2, tlast2, tstep2, rab0_2, time2, rab2,ak2,
     4       zetaX1, zetaX2, rmin, tmin
      CHARACTER*10  s0_1, s1, s0_2, s2
      CHARACTER*100 inpf, inpr, out, ver, rates1, rates2
      DIMENSION s0_1(nspec), s0_2(nspec), rab0_1(nspec), rab0_2(nspec),
     1          s1(nspec+2), s2(nspec+2), time1(ntime), time2(ntime),
     2          ak1(nreac), ak2(nreac), rab1(nspec+2,ntime), 
     3          rab2(nspec+2,ntime)

C Local variable(s):
      INTEGER i, j, k, nlen, is, js, nvis, nfiles, ipon, nait, ks, 
     1  itmin, istart, iend
      REAL*8 eps, rabr_max, rmax, rtol, aver, r1, r2, avera, r1a, r2a, 
     1  rmaxa
      CHARACTER*4 ait
      CHARACTER*10  specia, vispecies
	LOGICAL next
      DIMENSION rabr_max(nspec+2), is(nspec+2), js(nspec+2), 
     1  vispecies(nspec+2), avera(nspec), r1a(nspec), r2a(nspec),
     2  rmaxa(nspec)

C Current version of the code:
      PARAMETER (ver = ' version 1.4 (20.11.2009)')

C Data:
      REAL*8 aMp
      DATA  aMp /1.6724D-24/ 
	     
C Formats:
 100  FORMAT (A80)
 102  FORMAT (A10)
 105  FORMAT (D9.3)
 110  FORMAT (0PF5.0)
 115  FORMAT (1PE9.2)
 120  FORMAT (I4)
 125  FORMAT (A10,1X,D9.3)
 126  FORMAT ((9A10),:)
 130  FORMAT ('#',79('-'))
 135  FORMAT ('#',1X,'Species',8X,'Time',15X,'Abundances',10X,'n1/n2',
     &     10X,'<n1/n2>')
 140  FORMAT (3X,A10,5(5X,1PE9.3))

C------------------------------------------------------------------------------
C Read input data:
C------------------------------------------------------------------------------      
      open (10, file='input.compare')     
      READ (10,*)inpf,inpr,istart,iend,out,nvis,(vispecies(i),i=1,nvis)
     1 ,rmin, tmin
	close(10)
	
      nfiles = iend - istart + 1	
	
c Initialize output parameters:
      do i = 1, nvis
	  r1a(i) = 0.0d0
	  r2a(i) = 0.0d0
	  avera(i) = 0.0d0
	  rmaxa(i) = 0.0d0
	enddo

C Open output file 'out':
      CALL len_tri2(out,100,nlen)
      OPEN (unit=09,file=out(1:nlen),status='unknown',access='append')  
      REWIND 09

C Start loop by input files:
      DO ipon = istart, iend, 1

        IF (ipon.LE.9) THEN
          WRITE (ait,'(I1)') ipon
          nait = 1
        ELSE IF (ipon.LE.99) THEN
          WRITE (ait,'(I2)') ipon
          nait = 2
        ELSE IF (ipon.LE.999) THEN
          WRITE (ait,'(I3)') ipon
          nait = 3
        ELSE
          WRITE (ait,'(I4)') ipon
          nait = 4
        END IF

C Open input file 'inpf':
      CALL len_tri2(inpf,100,nlen)
      OPEN (unit=08,
     &  file=inpf(1:nlen)//'_'//ait(1:nait)//'.idl.0',
     &  status='old',access='sequential')   

C Read information from 'inpf':
        READ (08,*)
        READ (08,*)        
        READ (08,100) rates1
        READ (08,*) iter
        READ (08,*) 
        READ (08,*) 
        READ (08,110) tempg1
        READ (08,115) rhog1
        READ (08,115) rhod1
        READ (08,115) mdmg1 
        READ (08,115) grain1
        READ (08,*) G01
        READ (08,*) AvSt1
        READ (08,*) AvIS1
        READ (08,*) zeta1
        READ (08,*) zetaX1
        READ (08,115) albedo_UV1
        READ (08,115) tlast1
        READ (08,115) tstep1
        READ (08,120) ns0_1
        READ (08,*) (s0_1(i), rab0_1(i), i = 1, ns0_1)
        READ (08,*) ny1
        READ (08,126) (s1(i), i = 1, ny1)
        READ (08,*) nt1
        READ (08,*) (time1(i), i = 1, nt1)  
        READ (08,*) nr1
c        READ (08,*) (ak1(i), i = 1, nr1)  
        READ (08,'(10D12.5)') (ak1(i), i = 1, nr1)  
c        READ (08,'(10D23.16)') ((rab1(i,j), j = 1, nt1), i = 1, ny1)
        READ (08,'(10D12.5)') ((rab1(i,j), j = 1, nt1), i = 1, ny1)
C Close 'inpf':
      CLOSE (08)

C Open input file 'inpr':
      CALL len_tri2(inpr,100,nlen)
      OPEN (unit=08,
     &  file=inpr(1:nlen)//'_'//ait(1:nait)//'.idl',
     6  status='old',access='sequential')   
	
C Read information from 'inpr':
        READ (08,*)
        READ (08,*)        
        READ (08,100) rates2
        READ (08,*) 
        READ (08,*) 
        READ (08,*) 
        READ (08,110) tempg2
        READ (08,115) rhog2
        READ (08,115) rhod2
        READ (08,115) mdmg2
        READ (08,115) grain2
        READ (08,*) G02
        READ (08,*) AvSt2
        READ (08,*) AvIS2
        READ (08,*) zeta2
        READ (08,*) zetaX2
        READ (08,115) albedo_UV2
        READ (08,115) tlast2
        READ (08,115) tstep2
        READ (08,120) ns0_2
        READ (08,125) (s0_2(i), rab0_2(i), i = 1, ns0_2)
        READ (08,*) ny2
        READ (08,126) (s2(i), i = 1, ny2)
        READ (08,*) nt2
        READ (08,*) (time2(i), i = 1, nt2)  
        READ (08,*) nr2
c        READ (08,*) (ak2(i), i = 1, nr2)  
        READ (08,'(10D12.5)') (ak2(i), i = 1, nr2)  
c        READ (08,'(10D23.16)') ((rab2(i,j), j = 1, nt1), i = 1, ny1)
        READ (08,'(10D12.5)') ((rab2(i,j), j = 1, nt2), i = 1, ny2)
C Close 'inpr':
      CLOSE (08)

C..............................................................................
C I. Compare input parameters (must be identical!):
C..............................................................................
      eps = 1.0D-05

C Gas temperatures:
        IF (DABS(1.0D0-tempg1/tempg2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'T1= ',tempg1,' T2= ',tempg2
          STOP
        END IF
C Gas densities:
        IF (DABS(1.0D0-rhog1/rhog2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'rho1= ',rhog1,' rho2= ',rhog2
          STOP
        END IF
C Dust densities:
        IF (DABS(1.0D0-rhod1/rhod2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'rho1= ',rhod1,' rho2= ',rhod2
          STOP
        END IF
C Dust-to-gas ratios:
        IF (DABS(1.0D0-mdmg1/mdmg2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'rho1= ',mdmg1,' rho2= ',mdmg2
          STOP
        END IF
C Particle radii:
        IF (DABS(1.0D0-grain1/grain2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'rho1= ',grain1,' rho2= ',grain2
          STOP
        END IF
C Total time coarses of the evolution:
        IF (DABS(1.0D0-tlast1/tlast2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'tlast1= ',tlast1,' tlast2= ',tlast2
          STOP
        END IF
C First time steps:
        IF (DABS(1.0D0-tstep1/tstep2).GT.eps) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'tstep1= ',tstep1,' tstep2= ',tstep2
          STOP
        END IF
C Amount of taken time steps:
        IF (nt1.NE.nt2) THEN
          WRITE (*,*) 'Different input parameters:'
          WRITE (*,*) 'nt1= ',nt1,' nt2= ',nt2
          STOP
        END IF
C Calculated reaction rates:        
       if ((ny1.EQ.ny2).and.(nr1.EQ.nr2)) then
         do i = 1, nr1
           if (dabs(ak1(i)-ak2(i))/(ak1(i)+ak2(i)+1.0d-99).GT.eps) then
             write (*,*) 'Different reaction rates:'
             write (*,*) i,' ',ak1(i),' ',ak2(i)
             stop
           endif
         enddo
       endif


C..............................................................................
C II. Compare abundances of species:
C..............................................................................

      if (ipon.eq.istart) then

C Find abundances of the reduced species set in the full one:
        DO i = 1, ny2
          DO j = 1, ny1
            IF (s2(i).EQ.s1(j)) THEN
              js(i) = j
              exit
            END IF
          END DO 
        END DO

C Restrict search by the lowest chosen time moment:
        itmin = 0
        DO i = 1, nt1
	    if (time1(i).ge.tmin) then
		  itmin = i
	      exit
	    endif
	  ENDDO

      endif

C Start a loop by species to be compared:
        i = 0
   10   CONTINUE
        i = i + 1
	  if (i.gt.ny2) goto 20
          specia = s2(i)

C Check if this name in the list of species to be compared:
          next = .true.
          do j = 1, nvis
	      if (specia.eq.vispecies(j)) then
		    next=.false.
		    ks = j 
	        exit
	      endif
	    enddo

	    if (next) goto 10

            rmax = 0.0D+0 
	    aver = 0.0d0 
	    
C Start a loop by time steps: 
              DO j = itmin, nt1
                rab1(js(i),j) = dabs(rab1(js(i),j))+
     &                 1.0d-99*(0.7D0*rhog1/aMp)
                rab2(i,j) = dabs(rab2(i,j))+1.0d-99*(0.7D0*rhog2/aMp)                
	        r1=rab1(js(i),j)/(0.7D0*rhog1/aMp)
                r2=rab2(i,j)/(0.7D0*rhog2/aMp)
cd                print *,r1,r2,rab1(js(i),j),rab2(i,j)
                rtol = r2/r1
	          if (rtol.lt.1.0d0) rtol=1.0d0/rtol
	          if ((r1.le.rmin).or.(r2.le.rmin)) rtol=1.0d0
	          aver = aver + rtol
                IF (rtol.GT.rmax) THEN
	            rmax = rtol
	            k = j
	          END IF
              END DO

            aver = aver/(nt1-itmin+1)
            r1=rab1(js(i),k)/(0.7D0*rhog1/aMp)
            r2=rab2(i,k)/(0.7D0*rhog2/aMp)
            avera(ks) = avera(ks)+aver
	      r1a(ks) = r1a(ks)+r1
	      r2a(ks) = r2a(ks)+r2
	      rmaxa(ks) = dmax1(rmax,rmaxa(ks))

	    goto 10

   20   CONTINUE  
   

C Close loop by input files:
      ENDDO      

C Write results to the 'out':
      DO i = 1, nvis
        avera(i) = avera(i)/nfiles
	  r1a(i) = r1a(i)/nfiles
	  r2a(i) = r2a(i)/nfiles
	  rtol = 1.0d0
	  if ((r1a(i).gt.0.0d0).and.(r2a(i).gt.0.0d0)) then
	    rtol = r1a(i)/r2a(i)
	  endif  
	  if (rtol.lt.1.0d0) rtol=1.0d0/rtol
        WRITE (09,140) vispecies(i), r2a(i), r1a(i), rmaxa(i), avera(i), 
     $      rtol
      ENDDO
         
C Close all files:
      CLOSE (09)

C Exit:
      END
C..............................................................................
C
C This subroutine returns indices of the elements of a given array, ordered by
C decreasing.
C
C..............................................................................
C
C Version 1.0 (13/12/2001)
C
C..............................................................................
C
C Input parameter(s):
C
C array(1:na)  == a given array of values to be ordered,
C
C..............................................................................
C
C Output parameter(s):
C
C index(1:na)  == indices of the elements, have ordered by decreasing
C
C..............................................................................
      SUBROUTINE maxarr(na,array,index)
      IMPLICIT NONE
C Global variable(s):
      INTEGER na, index
      REAL*8 array
      DIMENSION array(na), index(na)
C Local variable(s):
      INTEGER i, j, imax, itmp
      REAL*8 rmax
C Initialization of 'index(1:na)':
       DO i = 1, na
         index(i) = i
       END DO
C Start loop of ordering the elements by decreasing:
        DO i = 1, na - 1
          rmax = array(i)
            imax = i
              DO j = i+1, na
                IF (DABS(array(j)).gt.DABS(rmax)) THEN
                  rmax = array(j)
                    imax = j
                      END IF
                      END DO
C Change positions of the ith and maximal_i elements:
                    rmax = array(i)
                  array(i) = array(imax)
                array (imax) = rmax
              itmp = index(i)
            index(i) = index(imax)
          index(imax) = itmp            
        END DO
C Exit:
      RETURN
      END   
C..............................................................................
C
C This subroutine returns a length of input character without final spaces.
C
C..............................................................................
C
C Version 1.0 (25/06/2001)
C
C..............................................................................
C
C Input parameter(s):
C
C x          == a input character,
C
C ixsize     == size of 'x',
C
C..............................................................................
C
C Output parameter(s):
C
C nlen       == a result length
C 
C..............................................................................
      SUBROUTINE len_tri2(x,ixsize,nlen)
      IMPLICIT NONE
C Global variable(s):
      INTEGER ixsize, nlen
      CHARACTER*132 x
C Local variable(s):
      INTEGER i
      CHARACTER*1 space
C Invert loop to search first non-space character from end of "x":      
        i = ixsize
10        CONTINUE
            space = x(i:i)  ! read i letter from the end of 'x',
              IF (space.ne.' ') GOTO 20  ! right length found, exit,
                i = i-1
              IF (i.eq.0) GOTO 20  ! 'x' is empty, exit,
            GOTO 10
20        CONTINUE      
C Define result:
        nlen = i
C Exit:
      RETURN
      END          

