C..............................................................................
C
C Program to model the chemical evolution under specified initial and physical 
c conditions (osu_2008_03 modification)
C
C..............................................................................
C
C copyright (c) 2002-2003, Dmitry Semenov, AIU Jena
C copyright (c) 2004-2008, Dmitry Semenov, MPIA Heidelberg
C
C..............................................................................
C Bug fixes:
C
C 5.1 -> 6.0: update to the modified rate approach
C 6.0 -> 6.2: New MRE with Bessel-like distribution (Biham & Lipstat 2002)
C 6.2 -> 6.3: Updated photorates can be used now (van Dishoeck 2006)
C 6.3 -> 6.4: "Shell" parallelization is added (through getarg & bash script)
C 6.4 -> 6.5: Now intensity of UV ISRF can be set to arbitary value
C 6.6: Corrected branching of the surface reactions, new implementation of
C modified rate approach
C *** This version is for a slightly modified OSU_2008_03 network ***
C..............................................................................
C
C Input parameter(s):  file: 0io.inp
C
C specs       == the name of a file contains names of chemical elements 
C                involved in a set of chemical reactions,
C
C rates       == the name of a file contains a system of chemical 
C                reactions for species from 'specs',
C
C out         == a name of output file contains calculated abundances,
C
C istart      == starting grid point,
C
C iend        == final grid point,
C
C rftype      == type of the radiation field: ISRF/TTau/Herb
C 
C Lya_part    == part of the UV flux from the Lyman alpha line
C
C UV_yield    == UV photodesorption yield (~10^-5 -- 10^-3)
C
C upho_file   == file name with updated photorates,
C
C UV_ISRF     == intensity of UV ISRF in units of the Draine's G-factor,
C
C scale_CRP   == a factor to scale the standard CRP ionization rate,
C
C zetaRN      == ionization rate due to decay of short-living radionuclides,
C
C scale_eD    == a factor to scale binding energies of surface molecules (except of H, H2, and D)
C
C input parameter(s):  file: 1environ_0d.inp
C
C Nr             == number of radial grid points,
C
C Nz             == number of vertical grid points,
C
C Rs(Nz)      == radial disk points (AU),
C
C Zs(Nz)      == vertical disk points (AU),
C 
C T(Nz)       == disk temperature [K],
C
C rho(Nz)     == disk density [g/cm^3],
C
C G0(Nz)      == stellar  rad. field in unit of the interstellar one,
C
C AvSt(Nz)    == a visual extinction, mag., in the direction on the star,
C
C AvIS(Nz)    == a visual interstellar extinction, mag.,
C
C ZetaCR(Nz)  == cosmic ray ionization rate,
C
C ZetaX(Nz)   == X-ray ionization rate,
C
C fH2(IS/St)  == shielding factor for H2 (Interstellar/Stellar components),
C
C fCO(IS/St)  == shielding factor for CO (Interstellar/Stellar components),
C
C aKdif(Nz)   == diffusion coefficient (in cm/s)
C
C input parameter(s):  file: 2times.inp
C
C tlast       == a time of the evolution of the system [years],
C
C tfirst      == first time step [years],
C
C nstep       == a number of time steps,
C
C input parameter(s):  file: 3abunds.inp
C
C nfrc        == dimension of initial values array 'frc',
C
C frc(1:nfrc) == initial amount of species 'y0(1:nfrc)' relative to H2,
C
C input parameter(s):  file: 4toleran.inp
C
C relerr      == a relative error of the calculations,
C
C abserr      == an absolute error of the calculations,
C
C
C input parameter(s):  file: 5surfchem.inp
C
C grain       == grain radius [cm],
C
C rhod        == grain density [g/cm^3],
C
C mdmg        == dust-to-gas mas ration (usually 1%),
C
C edconv      == ratio between desorption and diffusion energies,
C
C surf_des    == diff. to des. energy ratio, tunneling (yes|no),
C
C surf_method == method to handle surface chemistry modeling,
C
C material    == grain mantle composition,
C
C stoch_lim   == the lowest surface population when stochstic effects
C                become important,
C
C sprod       == relative fraction of products of surface reactions remaining
C                on dust surfaces upon formation  
C
C..............................................................................
C
C Output file(s):
C
C out.idl     == out written in a special format (absolute abundances!)
C
C..............................................................................
C
C Global parameter(s):
C
C nreac       == maximal amount of chemical reactions to be read,
C
C nspec       == maximal amount of chemical species to be considered,
C
C ntime       == maximal amount of taken time steps,
C 
C..............................................................................
C
C Common block(s):
C
C..............................................................................
C
C BL1:
C
C ns           == amount of species,
C
C s(ns)        == a species set,
C
C nre          == amount of reactions in the input file,
C
C index(nre)    == indices of the corresponding chemical reactions,
C
C r1(nre)       == names of the first reactants,
C
C ir1(nre)      == array of indices of 'r1' species in extended species set 's',
C
C r2(nre)       == names of the second reactants,
C
C ir2(nre)      == array of indices of 'r2' species in extended species set 's',
C
C p1(nre)       == names of the first products, 
C
C ip1(nre)      == array of indices of 'p1' species in extended species set 's',
C
C p2(nre)       == names of the second products, 
C
C ip2(nre)      == array of indices of 'p2' species in extended species set 's',
C
C p3(nre)       == names of the third products, 
C
C ip3(nre)      == array of indices of 'p3' species in extended species set 's',
C
C p4(nre)       == names of the fourth products, 
C
C ip4(nre)      == array of indices of 'p4' species in extended species set 's',
C
C p5(nr)        == names of the 5-th products, 
C
C ip5(nr)       == array of indices of 'p5' species in extended species set 's',
C
C alpha(nre)    == first components of the rate coeffs.,
C
C beta(nre)     == second components of the rate coeffs.,
C
C gamma(nre)    == third components of the rate coeffs.,
C
C ak(nre)       == rate coefficients, computed from 'alpha', 'beta' and 'gamma',
C
C ak_dH2        == rate coefficient of H + H|grain -> H2,
C
C..............................................................................
C
C Important variable(s):
C
C s(1:ns)     == an array with the names of species, 
C
C y(1:ns)     == an array with calculated number densities of species,
C
C gdens       == number density of hydrogen nucleii,
C
C ddens       == number density of dust grains,
C
C Stick       == sticking probability of species to be adsorbed on the dust,
C
C Stick_H     == sticking probability of H to be adsorbed on the dust,  
C
C Cion        == sticking probability of ions corrected for Coulomb attraction,
C
C Vth         == a thermal velocity for a chosen species,
C
C St          == a sticking probability for a chosen species
C
C..............................................................................
C
C Used function(s) (alphabetically): aweight
C
C Used subroutine(s) (alphabetically): abunds, ispecies, len_tri2, reads, readr
C 
C..............................................................................
      PROGRAM diskchemosu08
      IMPLICIT NONE

C Initialization of common blocks:
      INCLUDE "Fcn.h"
      INCLUDE "constants.h"      

C Global variable(s):
      INTEGER nfrc, nstep, istart, iend, ncpu, icpu
      REAL*4 dtime0, dtime1, dtime2
      REAL*8 tlast, tfirst, relerr, abserr, frc, Rs, Zs, rhod, UV_yield,
     1  T, rho, G0, AvST, AvIS, yy, ZetaCR, ZetaX, dfact, Stick,Stick_H, 
     2  Cion, Vth, aweight, St, ak_photo, ak_therm, ak_crp, tcrp, mdmg,
     3  fh2_IS, fh2_St, fco_IS, fco_St, grain, abundances, Surf,
     4  Lya_part, sprod, UV_ISRF, scale_CRP, zetaRN, scale_Ed
      CHARACTER*4 surf_des, rftype
      CHARACTER*7 gmat     
      CHARACTER*8 y0, rr1, rr2, pp1
      CHARACTER*132 specs, rates, upho_file, out, cmgtm, ver
      DIMENSION frc(nspec2),y0(nspec2), yy(nspec),
     1  abundances(nspec,ntime)

C Local variable(s):
      INTEGER i, j, l, nlen, Nr, Nz, nait, ipon, nlenr, iana, 
     1  iff, lx, is, li, lt, ides, ind0, ind1, jcpu
      REAL*8 fa, Hr, anu0, anu1, amur, des0, des1, edes, asg, 
     1  akbar, edconv, ed_mod, ans , mdmg_corr      
      CHARACTER*5 ait, siend
      CHARACTER*8 rrdes, sdes, sacc, sel_spec
      DIMENSION rrdes(nspec), edes(nspec), edconv(nspec),
     1 sdes(nreac),sacc(nreac) 
      LOGICAL sir

C Current version of the code:
      PARAMETER (ver = ' version 6.6 (26/11/2008)')

C Initialization of a driver routine to the integration of the ODEs:
      EXTERNAL drive, timeit

C Initialization of variables for 'timer':
      cmgtm = 'disk_chemistry,' //ver
cd	print *, cmgtm       

C Read total number of CPUs available for computations:
      call getarg(1,ait)
      read(ait,*) ncpu

C Read current CPU index:
      call getarg(2,ait)
      read (ait,*) icpu
      if (icpu.gt.ncpu) then
        write (*,*) 'Wrong CPU index: icpu>ncpu, stop'
        stop
      endif
C..............................................................................
C Read input data:
C 1) input & output file names:
C..............................................................................
      open (unit=01,file='0io.inp',status='old',
     &      access='sequential')
        read (01,*)  
        read (01,*) specs
        read (01,*) rates
        read (01,*) out
        
C Read species set from 'specs':
        CALL reads (specs)              
        CALL readr (rates)      

C Read position within the grid for which chemistry will be computed:
        read (01,*) istart
        if (istart.eq.0) istart = 1
        iend = 0
        read (01,*) siend
        if (siend.ne.'') read(siend,*) iend	

C Do we have better photorates?        
cd        read(01,*) rftype
cd        if (rftype(1:1).ne.' ') then
cd	      read(01,*) Lya_part
cd	      if (Lya_part.gt.0.0d0) then
cd	        print *, 'Photoreactions with the Ly_alpha radiation'
cd	        print *, 'are not yet included, stop!'
cd	        stop
cd	      endif
cd	      read (01,*) UV_yield
cd	      read(01,*) upho_file	      
cd	      CALL update_photorates(rftype,Lya_part,upho_file)	      
cd        endif
	     
	    read(01,*) UV_yield
        read(01,*) UV_ISRF
        read(01,*) scale_CRP
        read(01,*) zetaRN
        read(01,*) scale_Ed
        
      close (01)

C 3) Initial abundances:
      open (unit=01,file='3abunds.inp',status='old',
     &      access='sequential')
        read (01,*) 
        read (01,*) nfrc  
        do i = 1, nfrc
           read (01,*) y0(i), frc(i)  
cd           write(*,*) y0(i), frc(i)  
        end do
      close(01)

C 4) coarse of time evolution:
      open (unit=01,file='2times.inp',status='old',
     &      access='sequential')
        read (01,*)   
        read (01,'(D9.3)') tlast
                   tlast = tlast * year
        read (01,'(D9.3)') tfirst
                  tfirst = tfirst * year
        read (01,'(I3)') nstep
      close (01)

C 5) tolerance request:
      open (unit=01,file='4toleran.inp',status='old',
     &      access='sequential')
        read (01,*)   
        read (01,'(D9.3)') relerr
        read (01,'(D9.3)') abserr
      close (01)   
      
C 6) Dust grain parameters & surface chemistry properties:
      open (unit=01,file='5surfchem.inp',status='old',
     &      access='sequential')
        read (01,*)   
        read (01,*) grain
        read (01,*) rhod
        read (01,*) mdmg
        read (01,*) ed_mod
        if (ed_mod.gt.1.0d0) then
          print *, ' ed_mod must be  < 1, exit!'
          STOP
        endif        
        read (01,*) surf_des
        read (01,*) surf_method  
        read (01,*) gmat
        read (01,*) stoch_lim
        read (01,*) sprod
        if ((sprod.gt.1.0d0).or.(sprod.lt.0.0d0)) then
          print *, ' sprod must be between 0 and 1, exit!'
          stop
        endif
      close (01) 
      
      Sads = asg(grain,gmat)

C 2) Read environment's conditions:
      open (unit=01,file='1environ_0d.inp',status='old',
     &      access='sequential')
        read (01,*)
        read (01,*) Nr
        read (01,*) Nz
        if (iend.gt.(Nr*Nz)) then
          write(*,*) 'Wrong position of final grid cell is given:'
          write(*,*) 'iend = ',iend,' > Nr*Nz = ',Nr*Nz,', stop!'
          stop
        endif
        
        
C Correction factor for mass dust2gas ratio:
       mdmg = mdmg / 1.09754088749d0


C Run static chemistry over a 1D grid:
      dtime2 = 0.0
      open(10,file='times')

      jcpu = istart + icpu - 1
      
      DO i = 1, Nr*Nz, 1

        read (01,*) Rs, Zs, rho, T, G0, AvSt, AvIS, ZetaCR, ZetaX,
     &    fh2_IS, fco_IS, fh2_St, fco_St, Hr
     
        rho = rho * 1.09754088749d0 ! pure H to full gas particle density conversion

c Run chemistry in selected grid cells only:
        if ((i.ne.jcpu).or.(i.gt.iend)) goto 999
        jcpu = jcpu + ncpu
        
c Ionization by short-living radionuclides:
        ZetaCR = ZetaCR*scale_CRP + ZetaRN
cd        print *, ZetaCR

C==============================================================================
C Start the disk chemistry calculations:
C==============================================================================
        WRITE(*,*) 'Grid point N ', i

        dtime0 = 0.0
        CALL timeit(cmgtm, dtime0)

C------------------------------------------------------------------------------
C Start computations in the z-direction.
C Loop 2 by vertical points:
C------------------------------------------------------------------------------
C Gas particle density is:
         gdens = rho / aMp / amu / 1.09754088749d0
         print *, 'Gas particle density is ',gdens

C Amount of dust grains is:
         ddens = rho * mdmg/(4.0D0/3.0D0*PI*grain*grain*grain*rhod)/amu
         
C Slight rescaling to match the Bordeaux dust density value:
         
         print *, 'ndust = ', ddens/gdens, gdens/ddens
         print *, 'gtodn = ', 4.0d0/3.0d0*PI*rhod*grain**3/mdmg/aMp  

C Abundance of dust grains relative to the value given in HHL (1992):
cd         dfact= 2.66D-08 / ddens

C Compute a rate coefficient for production of H2 on
C dust surfaces (Hollenbach & McKee 1979):
cd         fa = 1D0 / (1D0 + 1D4*DEXP(-(6D2/T)))
cd         ak_dH2 = 3D-18*dsqrt(T)*fa /
cd     /   (1D0+4d-2*DSQRT(T+T)+2D-3*T+8D-6*T*T)

C Calculation of the sticking probabilities:
C Jones & Williams 1985 (Figure 2,3; the case of silicates):
         Stick = 1.0D0

C Hollenbach & McKee 1979:
         Stick_H = 1.0D0  !/ (1D0+4D-2*DSQRT(T+T)+2D-3*T+8D-6*T*T)

C Calculation of the long-range Coulomb attraction for ions:
         Cion = 1.0D0 + 1.671D-3 / grain / T

C------------------------------------------------------------------------------
C Calculation of the rate coefficients.
C Loop 3 by reactions:
C------------------------------------------------------------------------------
cd       sel_spec = 'gCH3N'
cd       open(60,file=sel_spec(1:len_trim(sel_spec))//'_rates.txt')
       
cd       print *, Avis
      ides = 0
      DO j = 1, nre
      
         ak(j) = 0.0d0      
         rr1 = r1(j)
         rr2 = r2(j)
         pp1 = p1(j)    
         
C a) Cosmic ray & X-ray ionization:
          IF (rr2.EQ.'CRP') THEN
            ak(j) = alpha(j)*zetaCR
            
cd          ELSE IF (rr2.EQ.'XRAYS') THEN
cd            ak(j) = alpha(j)*zetaX

C b) Photoreaction:
          ELSE IF (rr2.EQ.'PHOTON') THEN
          
cd            ak(j) = alpha(j)*dexp(-gamma(j)*AvSt)*G0+
cd     +              alpha_is(j)*dexp(-gamma_is(j)*AvIS*UV_ISRF)

            ak(j) = alpha(j)*dexp(-gamma(j)*AvIS)*G0
cd            write(61,*),rr1,' ',ak(j), alpha(j),gamma(j)

C H2 shielding:
cd          IF (rr1.EQ.'H2') ak(j)= alpha(j)*G0*fh2_St*dexp(-2.5d0*AvSt)+
cd     +        alpha_is(j)*UV_ISRF*fh2_IS*dexp(-2.5d0*AvIS)
	      
C CO shielding:
cd          IF (rr1.EQ.'CO') ak(j)= alpha(j)*G0*fco_St+
cd     +       alpha_is(j)*UV_ISRF*fco_IS

C c) Cosmic ray induced photoionization:
          ELSE IF (rr2.EQ.'CRPHOT') THEN
cd            ak(j) = alpha(j)*(zetaCR+zetaX)  !/(1.0D0-albedo_UV)

            ak(j) = alpha(j)*zetaCR  

C d) Dissociative recombination:
C Umebayashi & Nakano 1980 (Stick for ions ~1):
C Ions plus negatively charged grains:
          ELSE IF (rr2.EQ.'G-') THEN     
cd             Vth = DSQRT(8.0D0*ak_B*T/PI/aweight(rr1)/aMp)
             ak(j) = alpha(j)*(T/300.0D0)**beta(j)/ddens*gdens !alpha(j)*PI*grain*grain*Vth*Stick*Cion
             
C Ions plus neutral grains:
cd           ELSE IF (rr2.EQ.'G0') THEN     
cd             Vth = DSQRT(8.0D0*ak_B*T/PI/aweight(rr1)/aMp)
cd             ak(j) = alpha(j)*PI*grain*grain*Vth*Stick

C e) Collisions of the electrons with the neutral grains:
           ELSE IF (rr1.EQ.'G0') THEN     
cd             Vth = DSQRT(8.0D0*ak_B*T/PI/aMe)
             ak(j) = alpha(j)*(T/300.0D0)**beta(j)/ddens*gdens !PI*grain*grain*Vth !*1.329D0*DEXP(-T/20.0D0)
             
cd             print *, 'e- + G0: ',ak(j)

C f) Collisions of the electrons with the positively charged grains:
cd           ELSE IF (rr1.EQ.'G+') THEN     
cd             Vth = DSQRT(8.0D0*ak_B*T/PI/aMe)
cd             ak(j) = PI*grain*grain*Vth*1.329D0*DEXP(-T/20.0D0)*Cion

C g) Adsorption of the neutrals:
           ELSE IF (rr2.EQ.'FREEZE') THEN
             St = Stick
             IF (rr1.EQ.'H') St = Stick_H
             
             Vth = DSQRT(8.0D0*ak_B*T/PI/aweight(rr1)/aMp)                         
             ak(j) = alpha(j)*PI*grain*grain*Vth*ddens*St
	         
cd	         IF (rr1.EQ.'C3H') print *, 'C3H acc = ', ak(j) 
cd	         IF (rr1.EQ.'C3H') print *, 'C3H mass = ', aweight(rr1) 
	         
cd             IF (rr1.EQ.'H') ak(j)=DMAX1(0.0D0,ak(j)-2.0D0*ak_dH2)

C Save a copy of the adsorption rates in the common block:             
             Racc(j) = ak(j)
             sacc(j) = pp1

C h) Desorption processes:
C  Thermal desorption:
           ELSE IF (rr2.EQ.'DESORT') THEN

C Initialization of diffusion energy value according to the chosen
C model 'surf_des':           
             ides = ides + 1
             rrdes(ides) = rr1
             edes(ides) = gamma(j)*scale_Ed
             if (surf_des(1:4).eq.'HASG') then
               edconv(ides) = 0.3d0
             else if (surf_des(1:4).eq.'RUFL') then
               edconv(ides) = 0.77d0
             else
               print *, 'Wrong input parameter, stop: ',surf_des
               stop
             endif           
           
           
           anu0 = DSQRT(ans(gmat)*ak_B/aMp*gamma(j)*scale_Ed/PI/PI/
     &         aweight(rr1))


C Thermal desorption:
	     ak_therm = anu0*DEXP(-gamma(j)*scale_Ed/T) 
	     
	       
             ak(j) = ak_therm
cd             ak(j) = 0.0d0
             
C Save a copy of the desorption rates in the common block:                          
             Rdes(j) = ak(j)
             sdes(j) = rr1   
             
cd           if (rr1.eq.'gH2') print *, 'gH2 thdes = ',ak(j)  
cd           if (rr1.eq.'gH2') print *, 'gH2 nu0 = ',anu0  
cd           if (rr1.eq.'gH2') print *, 'gH2 ED = ',-gamma(j)*scale_Ed/T  
	       
C  CRP-induced desorption:
           ELSE IF (rr2.EQ.'DESORC') THEN

C Initialization of diffusion energy value according to the chosen
C model 'surf_des':           
             ides = ides + 1
             rrdes(ides) = rr1
             edes(ides) = gamma(j)*scale_Ed
             if (surf_des(1:4).eq.'HASG') then
               edconv(ides) = 0.3d0
             else if (surf_des(1:4).eq.'RUFL') then
               edconv(ides) = 0.77d0
             else
               print *, 'Wrong input parameter, stop: ',surf_des
               stop
             endif           
           
           
           anu0 = DSQRT(ans(gmat)*ak_B/aMp*gamma(j)*scale_Ed/PI/PI/
     &         aweight(rr1))	       

C CRP desorption:
C Le'ger et al. 1985 (CRP heating of 1000A grain, Eq. 1a):
             tcrp =   70.0d0 !(4.36D5+T*T*T)**0.3333333333d0
!             ak_crp = 2.431D-2*zetaCR*anu0*DEXP(-gamma(j)*scale_Ed/tcrp)

             ak_crp = 3d-19*(zetaCR/1.3d-17)*anu0*DEXP(-gamma(j)/tcrp)

             ak(j) = ak_crp   !*1.004d0
             
cd             if (rr1.eq.'gCH3N') ak(j) = 9.833E-25
cd             if (rr1.eq.'gC2H2N') ak(j) = 0.0E0
cd             if (rr1.eq.'gCH') ak(j) = 6.852E-13
cd             if (rr1.eq.'gH2') print *, 'gH2 CRPdes= ',ak(j)   != 1.133E-07
cd             

cd             ak(j) = 0.0d0
             
C Save a copy of the desorption rates in the common block:                          
             Rdes(j) = ak(j)
             sdes(j) = rr1 
             
cd      write(44,*) rr1,' ',rr2,' ',p1(j),' ',ak(j)

C i) Dust surface reaction:
           ELSE IF (rr2(1:1).EQ.'g') THEN 
           
cd             ak(j) = alpha(j)*dfact

C Initialize values of the diffusion energy for the reactants:           
             call ispecies(rr1,ides,rrdes,ind0)
             call ispecies(rr2,ides,rrdes,ind1)
             des0 = edes(ind0)
             des1 = edes(ind1)
             
C Compute their vibrational frequencies:    
             anu0 = DSQRT(ans(gmat)*ak_B/aMp*des0/PI/PI/aweight(rr1))
             anu1 = DSQRT(ans(gmat)*ak_B/aMp*des1/PI/PI/aweight(rr2))

C Compute diffusion rates of the reactants (thermal hopping and quantum tunneling for the model of HHL 1992):
             Rdiff0(j) = anu0*DEXP(-edconv(ind0)*des0/T)/Sads
             Rdiff1(j) = anu1*DEXP(-edconv(ind1)*des1/T)/Sads
                                                    
             if (surf_des(1:4).eq.'HASG') then
               if ((rr1(1:3).eq.'gH ').or.(rr1(1:4).eq.'gH2 ')) then
                 amur = aweight(rr1)*aMp
                 akbar=DEXP(-4D0*PI/hp*1D-8*
     &               DSQRT(2D0*amur*edconv(ind0)*des0*ak_B))
                 Rdiff0(j) = anu0*akbar/Sads
               endif  
               if ((rr2(1:3).eq.'gH ').or.(rr2(1:4).eq.'gH2 ')) then
                 amur = aweight(rr2)*aMp
                 akbar=DEXP(-4D0*PI/hp*1D-8*
     &               DSQRT(2D0*amur*edconv(ind1)*des1*ak_B))
                 Rdiff1(j) = anu1*akbar/Sads               
               endif    
             endif 
	     
C Compute efficiency of the exothermic reactions (assuming quantum tunneling for H and H2 for the model of HHL 1992):             
             akbar= DEXP(-gamma(j)*scale_Ed/T)
             if (surf_des(1:4).eq.'HASG') then
              if ((rr1(1:3).eq.'gH ').or.(rr1(1:4).eq.'gH2 ').or.
     &            (rr2(1:3).eq.'gH ').or.(rr2(1:4).eq.'gH2 ')) then
                 amur = aweight(rr1)*aweight(rr2)/
     /              (aweight(rr1)+aweight(rr2))*aMp
                 akbar=DEXP(-4D0*PI/hp*1D-8*
     &                 DSQRT(2D0*amur*gamma(j)*scale_Ed*ak_B))
               endif
             endif  
              
             if (pp1(1:1).eq.'g') then  
               ak(j) = sprod*akbar*(Rdiff0(j)+Rdiff1(j))/ddens*alpha(j)
               if (rr1.eq.rr2) ak(j) = ak(j)/2.0d0        
               ak_dens(j) = sprod*akbar/ddens*alpha(j)
               if (rr1.eq.rr2) ak_dens(j) = ak_dens(j)/2.0d0        
             else
               ak(j) = (1.0d0-sprod)*akbar*(Rdiff0(j)+Rdiff1(j))/ddens*
     &                 alpha(j)
               if (rr1.eq.rr2) ak(j) = ak(j)/2.0d0        
               ak_dens(j) = (1.0d0-sprod)*akbar/ddens*alpha(j)   
               if (rr1.eq.rr2) ak_dens(j) = ak_dens(j)/2.0d0                       
             endif  

C Initialize indices of the accretion and desorption rates for the both
C reactants:           
             call ispecies(rr1,nre,sacc,iar1(j))
             call ispecies(rr2,nre,sacc,iar2(j))             
             call ispecies(rr1,nre,sdes,idr1(j))
             call ispecies(rr2,nre,sdes,idr2(j))

cd         write (57,'(I4, 1x, 1pE8.2)'), index(j), ak(j)

     


C j) Two-body reaction:
           ELSE 
             ak(j) = alpha(j)*(T/300.0D0)**beta(j)*dexp(-gamma(j)/T)
        END IF 


cd        sir=.false.
cd        if (r1(j).eq.sel_spec) sir = .true.
cd        if (r2(j).eq.sel_spec) sir = .true.
cd        if (p1(j).eq.sel_spec) sir = .true.
cd        if (p2(j).eq.sel_spec) sir = .true.
cd        if (p3(j).eq.sel_spec) sir = .true.
cd        if (p4(j).eq.sel_spec) sir = .true.
cd        if (p5(j).eq.sel_spec) sir = .true.
cdcd        sir = .true.
cd        if (sir) write(60,'(2(1X,A8),9X,4(1X,A8),2X,1pE13.7)') rr1,rr2,
cd     ,      pp1,p2(j),p3(j),p4(j),ak(j)
cdcd     ,      p2(j),p3(j),p4(j),p5(j),alpha(j),beta(j),gamma(j),ak(j)


C End loop 3 (reactions):
      END DO
                   
      close(60)             
                   
 	  CALL abunds(ns,s,nfrc,y0,frc,gdens,ddens,yy)

C Compute chemistry:
      CALL drive(ns,tfirst,nstep,yy,tlast,relerr,abserr,abundances)

C==============================================================================
C WRITE results to the output files:
C==============================================================================
C Open output file '*.idl':
        ipon = i
        IF (ipon.LE.9) THEN
          WRITE (ait,'(I1)') ipon
          nait = 1
        ELSE IF (ipon.LE.99) THEN
          WRITE (ait,'(I2)') ipon
          nait = 2
        ELSE  IF (ipon.LE.999) THEN
          WRITE (ait,'(I3)') ipon
          nait = 3
        ELSE  IF (ipon.LE.9999) THEN
          WRITE (ait,'(I4)') ipon
          nait = 4
        ELSE  
          WRITE (ait,'(I5)') ipon
          nait = 5
        END IF

C Open output file 'out':
        CALL len_tri2(out,132,nlen)
        OPEN (unit=20,file=out(1:nlen)//'_'//ait(1:nait)//'.out',
     &        status='unknown',access='append')
        REWIND 20        
        OPEN (unit=30,file=out(1:nlen)//'_'//ait(1:nait)//'.idl',
     &        status='unknown',access='append')
        REWIND 30

C 1) Write current parameters to the 'out//.idl':
        WRITE(30,'(A18)') '# ART input file: '
        WRITE(30,'(a80)') specs
        WRITE(30,'(a80)') rates        
        WRITE(30,'(I5)') ipon
        WRITE(30,'(1PE9.2)') Rs
        WRITE(30,'(1PE9.2)') Zs
        WRITE(30,'(0PF5.0)') T
        WRITE(30,'(1PE9.2)') rho
        WRITE(30,'(1PE9.2)') rhod
        WRITE(30,'(1PE9.2)') mdmg
        WRITE(30,'(1PE9.2)') grain
        WRITE(30,'(1PE9.2)') AvSt
        WRITE(30,'(1PE9.2)') AvIS
        WRITE(30,'(1PE9.2)') G0
        WRITE(30,'(1PE9.2)') ZetaCR
        WRITE(30,'(1PE9.2)') ZetaX
        WRITE(30,'(1PE9.2)') albedo_UV
        WRITE(30,'(1PE9.2)') tlast / year
        WRITE(30,'(1PE9.2)') tfirst / year
        WRITE(30,'(I4)') nfrc
        WRITE(30,'(A8,3x,D22.15)') (y0(j), frc(j), j = 1, nfrc)
        WRITE(30,*) ns
        WRITE(30,'((9(A8,2x)),:)') (s(j), j = 1, ns)
        write(30,*) nstep
        write(30,*) (times(j)/year, j = 1, nstep)
        WRITE(30,*) nre
        WRITE(30,'(10D12.5)') (ak(j), j = 1, nre)
        write(30,'(10D12.5)') ((abundances(j,l), l=1,nstep), j=1,ns)
	CLOSE(30)

C 2) Write current parameters to the 'out':
      li = ns
      lt = li+1
      iana =1
      write(20,20)
      write(20,75)
 75   format(21x,39('*'))
      write(20,84) lt-1
 84   format(20X,' **  CIRCUMSTELLAR ENVELOPE CHEMISTRY **',/,
     * 20X,' **          ',1I3,' SPECIES SET          **',/,
     * 20X,' **            F77 VERSION            **',/,
     * 20X,' **             31/03/2007            **')
      write(20,75)
      write(20,20)
 21   format(1x,' SPECIES CONTAINED IN SCHEME: ',//)
      write(20,22) (s(j),j=1,ns)
 22   format(8(1x,a8,2x))
      write(20,23)
 23   format(/)
      write(20,36) ns
 36   format(1x,' NUMBER OF VALID SPECIES USED = ',1i4)
      write(20,45) nre
 45   format(1x,' NUMBER OF VALID REACTIONS USED = ',1i6)
      write(20,23)
 20   format(//)

C Write input parameters:
      write(20,64)
 64   format(3x,' INITIAL VALUES: '/)
      WRITE(20,65) ipon, Rs, Zs, gdens, T, rho, 
     &     T, rhod, mdmg, grain, AvSt, AvIS, G0,
     &     (ZetaCR+ZetaX), albedo_UV, 
     &     times(1)/year, tlast/year, 0.0d0
 65   FORMAT(
     * 3X,' Grid point  = ',I5,/,
     * 3X,' Radius      = ',1PE11.3,' AU',/,
     * 3X,' Height      = ',1PE11.3,' AU',/,
     * 3X,' n(H+2H2)    = ',1PE11.3,' cm^(-3)',/,
     * 3X,' Tg          = ',0PF8.1,'K',/,
     * 3X,' rho_g       = ',1PE11.3,' g/cm^3',/,
     * 3X,' Td          = ',0PF8.1,'K',/,
     * 3X,' rho_d       = ',1PE11.3,' g/cm^3',/,
     * 3X,' Mdust/Mgas  = ',1PE11.3,/,
     * 3X,' grain size  = ',1PE11.3,' cm',/,
     * 3X,' Av(stellar) = ',1PE11.3,' mag.',/,
     * 3X,' Av(IS)      = ',1PE11.3,' mag.',/,
     * 3X,' G0(stellar) = ',1PE11.3,' G0(IS)',/,
     * 3X,' Zeta        = ',1PE11.3,/,
     * 3X,' albedo(UV)  = ',1PE11.3,/,
     * 3X,' Start time  = ',0PF9.0,' years',/,
     * 3X,' Finish time = ',0PF9.0,' years',/,
     * 3X,' Diff. coeff.= ',1PE11.3)
      WRITE(20,20)     
C Write calculated abundances:
      write(20,63)
63    format(3X,' CALCULATED ABUNDANCES: '/)
       is = 1
       iff = 6
32    write(20,41) (s(j),j=is,iff)
      write(20,76)
      do l = 1, nstep
         write(20,30) times(l)/year,(abundances(j,l)/gdens,j=is,iff)
      end do
30    format(1x,7(1pe11.3))
      write(20,41)(s(j),j=is,iff)
41    format(6x,'time',6x,6(1a8,3x))
      write(20,20)
      is=is+6
      iff=iff+6
C Interupt?
      if (iff.gt.ns) iff=ns
      if (is.ge.li) go to 38
      go to 32
C Last output statement:
38    lx=li
      lt=lt-1
76    format(1x,80('-'))

      CLOSE (20)
      
cd      do j = 1, ns
cd       write(55,'(A10,3x,1pe12.5)'),s(j),abundances(j,nstep)/gdens
cd      enddo
      
C Print used CPU time:
      call timeit(cmgtm,dtime1)            
      dtime0 = dtime1 - dtime0
	dtime2 = dtime2 + dtime0
cdima      WRITE (*,*) ' '
      WRITE (10,*) 'CPU time: ', dtime0,'s'

999   CONTINUE

      END DO

C Print used CPU time:
cdima      WRITE (*,*) ' '
      WRITE (*, *) ' => Total time: ', dtime2,'s'
      WRITE (10,*) ' => Total time: ', dtime2,'s'

C Close all:
      close (01)
      close (10)
      

C Exit:
      END
C..............................................................................
C
C This subroutine computes initial abundances of chemical species for a
C given species set.
C
C..............................................................................
C
C Version 1.0 (11/12/2001)
C
C..............................................................................
C
C Input parameter(s):
C
C y(1:ny)        == the names of non-conserved species,
C
C yr(1:nr)       == the names of chemical species from '3abunds.inp',
C
C fraction(1:nr) == fractions of the corresponding species 'yr',
C
C density        == amount of hydrogen nuclei,
C
C ddens          == amount of dust grains
C 
C..............................................................................
C
C Output parameter(s):
C
C y0(1:ny)      == computed initial abundances
C
C..............................................................................
      SUBROUTINE abunds(ny,y,n,yr,fraction,density,ddens,y0)
      IMPLICIT NONE

C Global variable(s):
      INTEGER ny, n
      CHARACTER*8 y, yr
      REAL*8 fraction, density, y0, ddens
      DIMENSION y(ny), y0(ny), yr(n), fraction(n)

C Local variable(s):
      INTEGER i, j, k
      DIMENSION k(n)

C Initialization:
        do i = 1, ny
          y0(i) = 0.0D0 
        end do 
        do i = 1, n
          k(i) = 0
        end do 

C Search of species 'yr' among species 'y':
      i = 0
C A loop by 'yr':
10    continue
        i = i + 1
        if (i.gt.n)  goto 40  ! checked all 'yr', exit,

C A loop by 'y':
        j = 0
20      continue
        j = j + 1
        if (j.gt.ny) goto 10
        
C Found a necessary species:
        if (yr(i).eq.y(j)) then
          k(i) = j
          goto 10  ! switch to the next species 'yr',
        end if

        goto 20  ! switch to the next species 'y',

40    continue

C Calculation of the initial abundances:
      do i = 1, n
	  if (k(i).NE.0) then
	    print *,'Initially present is ',y(k(i))
          y0(k(i)) = fraction(i)*density          
          if ((y(k(i)).EQ.'G0').and.(fraction(i).gt.0.999d0)) then
            y0(k(i)) = fraction(i)*ddens
          endif
        endif
      end do

C Exit:
      return
      end
C..............................................................................
C
C This subroutine returns index of a given species in a given array of species
C names.
C
C..............................................................................
C
C Version 1.0 (14/12/2001)
C
C..............................................................................
C
C Input parameter(s):
C
C species      == name of a given species,
C
C array(1:na) == a given array of species names,
C
C..............................................................................
C
C Output parameter(s):
C
C index      == index of 'species' among 'array(1:na)'
C
C..............................................................................
      SUBROUTINE ispecies(species,na,array,index)
      IMPLICIT NONE
      
C Global variable(s):
      INTEGER na, index
      CHARACTER*8 species, array
      DIMENSION array(na)
      
C Local variable(s):
      INTEGER i
      
C Start search of 'species' in 'array':
      i = 1
 10   CONTINUE
      IF (i.gt.na) GOTO 20      ! No 'species' in 'array', stop,
      IF (species.eq.array(i)) GOTO 30 ! Found 'species', exit,
         i = i + 1
         GOTO 10
C 'species' was not found in 'array', stop:
 20   CONTINUE
cd      write (56,*) species
      index = 0
      RETURN
C 'Species' was found in 'array', initialize 'index':
 30   CONTINUE
      index = i
C Exit:
      RETURN
      END
C..............................................................................
C
C This subroutine returns a length of input character without final spaces.
C
C..............................................................................
C

C Version 1.0 (25/06/2001)
C
C..............................................................................
C
C Input parameter(s):
C
C x          == a input character,
C
C ixsize     == size of 'x',
C
C nlen       == a result length
C 
C..............................................................................
      subroutine len_tri2(x,ixsize,nlen)
      implicit none
C global variable(s):
      integer ixsize
      character*(*) x
      integer nlen
C local variable(s):
      integer i
      character*1 space
C invert loop to search first non-space character from end of "x":      
       i = ixsize
10      continue
         space = x(i:i)  ! read i letter from the end of 'x',
         if (space.ne.' ') goto 20  ! right length found, exit,
         i = i-1
         if (i.eq.0) goto 20  ! 'x' is empty, exit,
         goto 10
20      continue      
C result:
       nlen = i
C exit:
      return
      end             
C..............................................................................
C
C This function gives the surface density of adsorption sites X2 for various
C dust materials (see Biham et al. 2001).
C
C..............................................................................
C
C Version 1.1 (21.10.2006)
C
C..............................................................................
C
C Input parameter(s):
C 
C material  == grian mantle composition
C
C..............................................................................
C
C Output parameter(s):
C
C ans  == surface density of sites X 2
C
C..............................................................................
      REAL*8 FUNCTION ans(material)
      IMPLICIT NONE

C Global variable(s):
      character*7 material

C Surface density of sites x 2:
      if (material.EQ.'OLIVINE') then
        ans = 2.0d0*1.5d15  !2.0d14  !sites cm^-2

      else if (material.EQ.'ACARBON') then
        ans = 2.0d0*5.0d13  !sites cm^-2

      else 
        write (*,*) 'ANS: Input material name is wrong: ', material
        stop
      endif

C Exit:
      RETURN
      END
C..............................................................................
C
C This function computes total number of surface sites on a spherical grain
C of a given radius, using surface density of sites of the relevant grain
C material
C
C..............................................................................
C
C Version 1.1 (21.10.2006)
C
C..............................................................................
C
C Input parameter(s):
C 
C radius    == grain radius in cm,
C
C material  == grain mantle composition
C
C..............................................................................
C
C Output parameter(s):
C
C ans  == total amount of surface sites available for adsorption
C
C..............................................................................
C
C Used functions: ans
C
C..............................................................................
      REAL*8 FUNCTION asg(radius,material)
      IMPLICIT NONE

C Pi:
      INCLUDE "constants.h"

C Global variable(s):
      real*8 radius
      character*7 material

C Surface density of sites:
      REAL*8 ans
      EXTERNAL ans

C Amount of adsorption sites on a grain:
      asg = 2.0d0*pi*radius*radius*ans(material)
cd      print *, 'asg=',asg

C Exit:
      RETURN
      END
      
C..............................................................................
C
C This subroutine updates the UV photorates.
C
C..............................................................................
C
C Version 1.0 (06/11/2007)
C
C..............................................................................
C
C Input parameter(s):
C
C rftype     == type of the input radiation field,
C
C Ly_alpha   == a part of the UV flux in th Lyman alpha line,
C
C input      == a name of a file with updated photorates
C
C..............................................................................
C
C Common block(s):
C
C..............................................................................
C
C BL1:
C
C ns           == amount of species,
C
C s(ns)        == a species set,
C
C nre          == amount of reactions in the input file,
C
C index(nre)    == indices of the corresponding chemical reactions,
C
C r1(nre)       == names of the first reactants,
C
C ir1(nre)      == array of indices of 'r1' species in extended species set 's',
C
C r2(nre)       == names of the second reactants,
C
C ir2(nre)      == array of indices of 'r2' species in extended species set 's',
C
C p1(nre)       == names of the first products, 
C
C ip1(nre)      == array of indices of 'p1' species in extended species set 's',
C
C p2(nre)       == names of the second products, 
C
C ip2(nre)      == array of indices of 'p2' species in extended species set 's',
C
C p3(nre)       == names of the third products, 
C
C ip3(nre)      == array of indices of 'p3' species in extended species set 's',
C
C p4(nre)       == names of the fourth products, 
C
C ip4(nre)      == array of indices of 'p4' species in extended species set 's',
C
C p5(nr)        == names of the 5-th products, 
C
C ip5(nr)       == array of indices of 'p5' species in extended species set 's',
C
C alpha(nre)    == first components of the rate coeffs.,
C
C beta(nre)     == second components of the rate coeffs.,
C
C gamma(nre)    == third components of the rate coeffs.,
C
C ak(nre)       == rate coefficients, computed from 'alpha', 'beta' and 'gamma',
C
C..............................................................................
C
C Used subroutines(s) (alphabetically): len_tri2
C
C..............................................................................
cd      SUBROUTINE update_photorates(rftype,Lya_part,input)
cd      IMPLICIT NONE
cd
cdC Global variable(s):
cd      REAL*8 Lya_part
cd      CHARACTER*4 rftype
cd      CHARACTER*80 input
cd
cdC Initialization of common blocks:
cd      INCLUDE "Fcn.h"
cd      
cdC Local variable(s):
cd      INTEGER i, j, nrupd, nlen
cd      REAL*8 rdata(7)
cd      CHARACTER*8 su, p1u, rr1, rr2, pp1
cd      LOGICAL update
cd
cdC Read input file:
cd      CALL len_tri2(input,80,nlen)
cd      OPEN (unit=10,file=input(1:nlen),status='old',
cd     &      access='sequential')
cd      rewind 10
cd     
cd     
cdcd      open (55,file='upd_pd.txt')
cdcd      open (56,file='upd_pi.txt')
cd     
cdC I. Photodissociation updates:     
cd      read (10,*)
cd      read (10,*)      
cd      read (10,*)      
cd      read (10,*) nrupd   
cd      
cdcd      write (55,*) rftype
cd      
cd      
cd      do i = 1, nrupd
cd      
cd        read (10,*) su, p1u, (rdata(j),j=1,7)
cdcd        write (55,*) su, p1u, (rdata(j),j=1,7)
cd        
cd        do j = 1, nre
cd        
cd          rr1 = r1(j)
cd          rr2 = r2(j)
cd          pp1 = p1(j)
cd        
cd          update=( (rr1.eq.su).and.(pp1.eq.p1u).and.
cd     .             (rr2(1:6).eq.'PHOTON') )
cd          if (update.and.(ion_reac(j).eq.0)) then
cd          
cdcd        write (55,*), r1(j),' => ',p1(j),' ',p2(j),' ',p3(j)
cdcd        write (55,*), 'Old rate: ',alpha(j),' ',gamma(j)                
cd          
cdC Initialize ISRF rates:
cd              alpha_is(j) = rdata(1)
cd              gamma_is(j) = rdata(5)
cd              if (Lya_part.ne.0.0d0) beta_is(j) = rdata(4)
cd              
cd            if (rftype.eq.'ISRF') then              
cd              
cd              alpha(j) = alpha_is(j)
cd              gamma(j) = gamma_is(j)
cd              beta(j) = beta_is(j)            
cd            
cd            else if (rftype.eq.'Herb') then
cd            
cd              alpha(j) = rdata(2)
cd              gamma(j) = rdata(6)
cd              if (Lya_part.ne.0.0d0) beta(j) = rdata(4)            
cd            
cd            else if (rftype.eq.'TTau') then
cd
cd              alpha(j) = rdata(3)
cd              gamma(j) = rdata(7)
cd              if (Lya_part.ne.0.0d0) beta(j) = rdata(4)            
cd            
cd            else 
cd            
cd              print *, 'PD: Wrong type of radiation field: ', rftype
cd              print *, 'stop!'
cd              stop
cd            
cd            endif
cd            
cdcd            write (55,*), 'New rate: ',alpha(j),' ',gamma(j)          
cd            
cd          
cd          endif
cd        
cd        enddo
cd        
cd      enddo
cd
cdC II. Photoionization updates:     
cd      read (10,*)
cd      read (10,*)      
cd      read (10,*) nrupd   
cd           
cdcd      write (56,*) rftype
cd      
cd      do i = 1, nrupd
cd      
cd        read (10,*) su, (rdata(j),j=1,7)
cdcd        write (56,*) su, (rdata(j),j=1,7)
cd        
cd        do j = 1, nre
cd        
cd          rr1 = r1(j)
cd          rr2 = r2(j)
cd          
cd          update=((rr1.eq.su).and.(rr2(1:6).eq.'PHOTON'))
cd          if (update.and.(ion_reac(j).eq.1)) then
cd          
cdcd        write (56,*), r1(j),' => ',p1(j),' ',p2(j),' ',p3(j)
cdcd        write (56,*), 'Old rate: ',alpha(j),' ',gamma(j)          
cd          
cdC Initialize ISRF rates:
cd              alpha_is(j) = rdata(1)
cd              gamma_is(j) = rdata(5)
cd              if (Lya_part.ne.0.0d0) beta_is(j) = rdata(4)
cd              
cd            if (rftype.eq.'ISRF') then              
cd              
cd              alpha(j) = alpha_is(j)
cd              gamma(j) = gamma_is(j)
cd              beta(j) = beta_is(j)            
cd                        
cd            else if (rftype.eq.'Herb') then
cd            
cd              alpha(j) = rdata(2)
cd              gamma(j) = rdata(6)
cd              if (Lya_part.ne.0.0d0) beta(j) = rdata(4)            
cd            
cd            else if (rftype.eq.'TTau') then
cd
cd              alpha(j) = rdata(3)
cd              gamma(j) = rdata(7)
cd              if (Lya_part.ne.0.0d0) beta(j) = rdata(4)            
cd            
cd            else 
cd            
cd              print *, 'PI: Wrong type of radiation field: ', rftype
cd              print *, 'stop!'
cd              stop
cd            
cd            endif
cd            
cdcd            write (56,*), 'New rate: ',alpha(j),' ',gamma(j)          
cd
cd          
cd          endif
cd        
cd        enddo
cd        
cd      enddo
cd
cdC Close all files:
cd      CLOSE (10)
cd      
cdcd      close (55)
cdcd      close (56)
cd      
cdC Exit:
cd      RETURN
cd      END      
C..............................................................................
C
C This function computes atomic weight of a given species.
C
C..............................................................................
C
C Version 1.0 (27/02/2002)
C
C..............................................................................
C
C Input parameter(s):
C 
C specs    == a name of species
C
C..............................................................................
C
C Output parameter(s):
C
C aweight  == computed atomic weight
C
C..............................................................................
      REAL*8 FUNCTION aweight(x)
      IMPLICIT NONE
C Global variable(s):
      CHARACTER*8 x

C Local variable(s):
      INTEGER i, first, last, n
      REAL*8 w1, w2
      CHARACTER*1 s1, tmp1, multi
      CHARACTER*2 s2, tmp2
      LOGICAL cond
      DIMENSION s1(7), w1(7), s2(6), w2(6)

C Initialization of the output:
        aweight = 0.0D0

C Initialization of atom's names:
        s1(1) = 'H'
        s1(2) = 'C'
        s1(3) = 'N'
        s1(4) = 'O'
        s1(5) = 'S'
        s1(6) = 'P'
        s1(7) = 'D'
        s2(1) = 'HE'
        s2(2) = 'FE'
        s2(3) = 'SI'
        s2(4) = 'NA'
        s2(5) = 'MG'
        s2(6) = 'CL'

C and their weights:
        w1(1) =  1.0d0       !1.00790E-00      
        w1(2) = 12.0d0       !1.20110E+01
        w1(3) = 14.0d0       !1.40067E+01
        w1(4) = 16.0d0       !1.59994E+01
        w1(5) = 32.0d0       !3.20660E+01
        w1(6) = 31.0d0       !3.09738E+01
        w1(7) = 2.0d0       !2.0*w1(1)
        w2(1) = 4.0d0       !4.00260E-00
        w2(2) = 56.0d0       !5.58470E+01
        w2(3) = 28.0d0       !2.80855E+01
        w2(4) = 23.0d0       !2.29898E+01
        w2(5) = 24.0d0       !2.43050E+01
        w2(6) = 35.0d0       !3.54527E+01

C Initialization of bourders of the species name:
      CALL len_tri2(x,8,last)

C First consider a special case of C10-bearing species:
        IF ((x(1:3).EQ.'C10').or.(x(1:4).EQ.'gC10')) THEN
          aweight = 120.00D+00
          RETURN
        END IF

C Other cases:       
        first = 1
        
C Adsorbed species:
        IF (x(first:first).EQ.'g') THEN
          first = 2
C Ions:
        ELSE IF (x(last:last).EQ.'+') THEN
          last = last - 1
          IF (x(last:last).EQ.'+') last = last - 1
        ELSE IF (x(last:last).EQ.'-') THEN
          last = last - 1
        END IF

C Divide species onto atomic constituents:
 10   CONTINUE
        IF (first.GT.last) GOTO 20 ! checked the species,  exit,
C If it is not a last character in the species:
        IF ((last-first).GE.1) THEN
          tmp1 = x(first:first)
            tmp2 = x(first:(first+1))
              cond = .true.
C Start a search among two-letter atoms:
                DO i = 1, 6
                  IF (tmp2.EQ.s2(i)) THEN
                  aweight = aweight + w2(i)
                first = first + 2
              cond = .false.
            END IF       
          END DO 
C Start a search among one-letter atoms:
          IF (cond) THEN
            DO i = 1, 7
              IF (tmp1.EQ.s1(i)) THEN
                multi = tmp2(2:2)
C Find a possible multiplicators:
                                 n = 1
                       IF (multi.EQ.'2') THEN
                                 n = 2
                            first = first + 2
                  ELSE IF (multi.EQ.'3') THEN
                                 n = 3
                            first = first + 2
                  ELSE IF (multi.EQ.'4') THEN
                                 n = 4
                            first = first + 2
                  ELSE IF (multi.EQ.'5') THEN
                                 n = 5
                            first = first + 2
                  ELSE IF (multi.EQ.'6') THEN
                                 n = 6
                            first = first + 2
                  ELSE IF (multi.EQ.'7') THEN
                                 n = 7
                            first = first + 2
                  ELSE IF (multi.EQ.'8') THEN
                                 n = 8
                            first = first + 2
                  ELSE IF (multi.EQ.'9') THEN
                                 n = 9
                            first = first + 2
C There is no a multiplicator:
                  ELSE
                            first = first + 1                    
                  END IF
                aweight = aweight + n * w1(i)
              END IF
            END DO
          END IF  

C Checking the last character:
        ELSE
          DO i = 1, 7
            IF (x(first:last).EQ.s1(i)) THEN
              aweight = aweight + w1(i)
              first = first + 1
            END IF
          END DO 
        END IF

C Switch to the next character:         
      GOTO 10   
 20   CONTINUE

C Exit:
cd      write(55,*) x,' ',aweight
      RETURN
      END
      